<?php
namespace JLTELMT;

use JLTELMT\Libs\Assets;
use JLTELMT\Libs\Helper;
use JLTELMT\Libs\Featured;
use JLTELMT\Inc\Classes\Recommended_Plugins;
use JLTELMT\Inc\Classes\Notifications\Notifications;
use JLTELMT\Inc\Classes\Pro_Upgrade;
use JLTELMT\Inc\Classes\Row_Links;
use JLTELMT\Inc\Classes\Upgrade_Plugin;
use JLTELMT\Inc\Classes\Feedback;
use JLTELMT\Inc\Addon\Master_Table;

/**
 * Main Class
 *
 * @master-table-for-elementor
 * Jewel Theme <support@jeweltheme.com>
 * @version     1.0.2
 */

// No, Direct access Sir !!!
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * JLT_Elementor_Master_Table Class
 */
if ( ! class_exists( '\JLTELMT\JLT_Elementor_Master_Table' ) ) {

	/**
	 * Class: JLT_Elementor_Master_Table
	 */
	final class JLT_Elementor_Master_Table {

		const VERSION            = JLTELMT_VER;
		
		const MINIMUM_PHP_VERSION = '5.4';

		const MINIMUM_ELEMENTOR_VERSION = '2.0.0';

		private static $instance = null;

		/**
		 * what we collect construct method
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function __construct() {
			$this->includes();
			add_action( 'plugins_loaded', array( $this, 'jlt_elementor_master_table_plugins_loaded' ), 999 );
			// Body Class.
			add_filter( 'admin_body_class', array( $this, 'jlt_elementor_master_table_body_class' ) );
			// This should run earlier .
			// add_action( 'plugins_loaded', [ $this, 'jlt_elementor_master_table_maybe_run_upgrades' ], -100 ); .


			// Add Elementor Widgets
			add_action( 'elementor/widgets/widgets_registered', [ $this, 'jlt_elementor_master_table_init_widgets' ] );

			//Body Class
			add_filter( 'body_class', [ $this, 'jlt_elementor_master_table_ea_body_class' ] );			
		}



		public function jlt_elementor_master_table_ea_body_class($classes){
			if ( !\Elementor\Plugin::$instance->preview->is_preview_mode() ) {
				$classes[] = 'master-table-for-elementor';
			}
			return $classes;
		}


		/**
		 * Check if Elementor Activated
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */

		public function is_elementor_activated( $plugin_path = 'elementor/elementor.php' ) {
			$installed_plugins_list = get_plugins();

			return isset( $installed_plugins_list[ $plugin_path ] );
		}


		/**
		 * Register Widget Type
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */

		public function jlt_elementor_master_table_init_widgets() {
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Master_Table() );	
		}



		/**
		 * plugins_loaded method
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function jlt_elementor_master_table_plugins_loaded() {
			
			$this->jlt_elementor_master_table_activate();


			// Check if Elementor installed and activated
			if ( ! did_action( 'elementor/loaded' ) ) {
				add_action( 'admin_notices', array( $this, 'jlt_elementor_master_table_admin_notice_missing_main_plugin' ) );
				return;
			}

			// Check for required Elementor version
			if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
				add_action( 'admin_notices', array( $this, 'jlt_elementor_master_table_admin_notice_minimum_elementor_version' ) );
				return;
			}

			// Check for required PHP version
			if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
				add_action( 'admin_notices', array( $this, 'jlt_elementor_master_table_admin_notice_minimum_php_version' ) );
				return;
			}			
		}

		/**
		 * Version Key
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public static function plugin_version_key() {
			return Helper::jlt_elementor_master_table_slug_cleanup() . '_version';
		}

		/**
		 * Activation Hook
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public static function jlt_elementor_master_table_activate() {
			$current_jlt_elementor_master_table_version = get_option( self::plugin_version_key(), null );

			if ( get_option( 'jlt_elementor_master_table_activation_time' ) === false ) {
				update_option( 'jlt_elementor_master_table_activation_time', strtotime( 'now' ) );
			}

			if ( is_null( $current_jlt_elementor_master_table_version ) ) {
				update_option( self::plugin_version_key(), self::VERSION );
			}

			$allowed = get_option( Helper::jlt_elementor_master_table_slug_cleanup() . '_allow_tracking', 'no' );

			// if it wasn't allowed before, do nothing .
			if ( 'yes' !== $allowed ) {
				return;
			}
			// re-schedule and delete the last sent time so we could force send again .
			$hook_name = Helper::jlt_elementor_master_table_slug_cleanup() . '_tracker_send_event';
			if ( ! wp_next_scheduled( $hook_name ) ) {
				wp_schedule_event( time(), 'weekly', $hook_name );
			}
		}


		/**
		 * Add Body Class
		 *
		 * @param [type] $classes .
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function jlt_elementor_master_table_body_class( $classes ) {
			$classes .= ' master-table-for-elementor ';
			return $classes;
		}

		/**
		 * Run Upgrader Class
		 *
		 * @return void
		 */
		public function jlt_elementor_master_table_maybe_run_upgrades() {
			if ( ! is_admin() && ! current_user_can( 'manage_options' ) ) {
				return;
			}

			// Run Upgrader .
			$upgrade = new Upgrade_Plugin();

			// Need to work on Upgrade Class .
			if ( $upgrade->if_updates_available() ) {
				$upgrade->run_updates();
			}
		}

		/**
		 * Include methods
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function includes() {
			new Assets();
			new Recommended_Plugins();
			new Row_Links();
			new Pro_Upgrade();
			new Notifications();
			new Featured();
			new Feedback();
		}



		public function jlt_elementor_master_table_admin_notice_missing_main_plugin() {
			$plugin = 'elementor/elementor.php';

			if ( $this->is_elementor_activated() ) {
				if ( ! current_user_can( 'activate_plugins' ) ) {
					return;
				}
				$activation_url = wp_nonce_url( 'plugins.php?action=activate&amp;plugin=' . $plugin . '&amp;plugin_status=all&amp;paged=1&amp;s', 'activate-plugin_' . $plugin );
				$message = sprintf( 'Master Table for Elementor requires <b>Elementor</b> plugin to be active. Please activate Elementor to continue.', 'master-table-for-elementor' );
				$button_text = esc_html__( 'Activate Elementor', 'master-table-for-elementor' );

			} else {
				if ( ! current_user_can( 'install_plugins' ) ) {
					return;
				}

				$activation_url = wp_nonce_url( self_admin_url( 'update.php?action=install-plugin&plugin=elementor' ), 'install-plugin_elementor' );
				$message = sprintf( esc_html__( 'Master Table for Elementor requires %1$s"Elementor"%2$s plugin to be installed and activated. Please install Elementor to continue.', 'master-table-for-elementor' ), '<strong>', '</strong>' );
				$button_text = esc_html__( 'Install Elementor', 'master-table-for-elementor' );
			}

			$button = '<p><a href="' . esc_url_raw( $activation_url ) . '" class="button-primary">' . esc_html( $button_text ) . '</a></p>';

			printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p>%2$s</div>', $message , $button );

		}

		public function jlt_elementor_master_table_admin_notice_minimum_elementor_version() {
			if ( isset( $_GET['activate'] ) ) {
				unset( $_GET['activate'] );
			}

			$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
				esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'master-table-for-elementor' ),
				'<strong>' . esc_html__( 'Master Addons for Elementor', 'master-table-for-elementor' ) . '</strong>',
				'<strong>' . esc_html__( 'Elementor', 'master-table-for-elementor' ) . '</strong>',
				self::MINIMUM_ELEMENTOR_VERSION
			);

			printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
		}

		public function jlt_elementor_master_table_admin_notice_minimum_php_version() {
			if ( isset( $_GET['activate'] ) ) {
				unset( $_GET['activate'] );
			}

			$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
				esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'master-table-for-elementor' ),
				'<strong>' . esc_html__( 'Master Addons for Elementor', 'master-table-for-elementor' ) . '</strong>',
				'<strong>' . esc_html__( 'PHP', 'master-table-for-elementor' ) . '</strong>',
				self::MINIMUM_PHP_VERSION
			);

			printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
		}

		/**
		 * Initialization
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function jlt_elementor_master_table_init() {
			$this->jlt_elementor_master_table_load_textdomain();
		}


		/**
		 * Text Domain
		 *
		 * @author Jewel Theme <support@jeweltheme.com>
		 */
		public function jlt_elementor_master_table_load_textdomain() {
			$domain = 'master-table-for-elementor';
			$locale = apply_filters( 'jlt_elementor_master_table_plugin_locale', get_locale(), $domain );

			load_textdomain( $domain, WP_LANG_DIR . '/' . $domain . '/' . $domain . '-' . $locale . '.mo' );
			load_plugin_textdomain( $domain, false, dirname( JLTELMT_BASE ) . '/languages/' );
		}
		
		
		

		/**
		 * Returns the singleton instance of the class.
		 */
		public static function get_instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof JLT_Elementor_Master_Table ) ) {
				self::$instance = new JLT_Elementor_Master_Table();
				self::$instance->jlt_elementor_master_table_init();
			}

			return self::$instance;
		}
	}

	// Get Instant of JLT_Elementor_Master_Table Class .
	JLT_Elementor_Master_Table::get_instance();
}